#include "p2p_utilities.h"
#include "gio/gnetworking.h"
#include "json/json.h"

namespace offcn
{
	static const std::string kCandidateTypes[] = { "host", "srflx", "prflx", "relay" };

	bool BuildCandidatesString(NiceAgent *agent, guint stream_id, std::string &candidates)
	{
		bool result = false;

		Json::Value message;
		Json::Value candArray;
		Json::StreamWriterBuilder writerBuilder;

		writerBuilder["commentStyle"] = "None";		//̶ôд
		writerBuilder["indentation"] = "   ";		//̶ôд

		gchar  *ufrag = NULL;
		gchar  *password = NULL;
		gchar  ipaddr[INET6_ADDRSTRLEN] = { 0 };
		guint  port = 0;
		std::string foundation;
		guint  priority = 0;
		std::string type;

		GSList *cands = NULL;
		GSList *item = NULL;

		if (!nice_agent_get_local_credentials(agent, stream_id, &ufrag, &password))
		{
			return false;
		}

		cands = nice_agent_get_local_candidates(agent, stream_id, 1);
		if (cands == NULL)
		{
			goto Error;
		}

		message["ufrag"] = ufrag;
		message["password"] = password;

		for (item = cands; item; item = item->next)
		{
			NiceCandidate *c = (NiceCandidate *)item->data;

			nice_address_to_string(&c->addr, ipaddr);
			port = nice_address_get_port(&c->addr);
			if (port == 0) continue;

			foundation = c->foundation;
			type = kCandidateTypes[c->type];
			priority = c->priority;

			//if (nice_address_ip_version(&c->addr) == 6) continue;

			//if (c->type != 1) continue;

			Json::Value item;
			item["ip"] = ipaddr;
			item["port"] = port;
			item["priority"] = priority;
			item["type"] = type;
			item["foundation"] = foundation;

			candArray.append(item);
		}

		message["candidates"] = candArray;

		candidates = Json::writeString(writerBuilder, message);

		result = true;

	Error:
		if (ufrag)
		{
			g_free(ufrag);
		}
		if (password)
		{
			g_free(password);
		}
		if (cands) g_slist_free_full(cands, (GDestroyNotify)&nice_candidate_free);

		return result;
	}
	bool ParseCandidatesString(std::string &candidates, unsigned int stream_id, std::string &ufrag, std::string &password, GSList **list)
	{
		Json::Value root;
		Json::CharReaderBuilder readerBuilder;

		const char *start_pos = candidates.c_str();

		std::string error;
		std::unique_ptr<Json::CharReader> const reader(readerBuilder.newCharReader());

		if (!reader->parse(start_pos, start_pos + candidates.length(), &root, &error))
		{
			return false;
		}

		if (!root["ufrag"] || !root["password"] || !root["candidates"])
		{
			return false;
		}

		ufrag = root["ufrag"].asString();
		password = root["password"].asString();

		Json::Value candList = root["candidates"];
		for (int i = 0; i < candList.size(); i++)
		{
			Json::Value item = candList[i];
			if (!item["type"] || !item["ip"] || !item["port"] || !item["priority"])
			{
				continue;
			}

			std::string type = item["type"].asString();
			std::string ip = item["ip"].asString();
			std::string foundation = item["foundation"].asString();
			unsigned short port = item["port"].asUInt();
			unsigned int priority = item["priority"].asUInt();

			NiceCandidateType cType = NICE_CANDIDATE_TYPE_HOST;
			for (unsigned int index = 0; index < G_N_ELEMENTS(kCandidateTypes); index++)
			{
				if (type.compare(kCandidateTypes[index]) == 0)
				{
					cType = (NiceCandidateType)index;
					break;
				}
			}

			NiceCandidate *cand = nice_candidate_new(cType);
			cand->component_id = 1;
			cand->stream_id = stream_id;
			cand->transport = NICE_CANDIDATE_TRANSPORT_UDP;
			cand->priority = priority;
			strcpy_s(cand->foundation, NICE_CANDIDATE_MAX_FOUNDATION, foundation.c_str());

			if (!nice_address_set_from_string(&cand->addr, ip.c_str()))
			{
				nice_candidate_free(cand);
				cand = NULL;
				continue;
			}
			nice_address_set_port(&cand->addr, port);

			*list = g_slist_prepend(*list, cand);
		}

		if (*list == NULL) return false;

		return true;
	}
}