#pragma once

#include <string>
#include <thread>

extern "C"
{
#include "nice.h"
}

namespace offcn
{
	class P2pWrapperObserver
	{
	public:
		virtual void OnGetLocalCandidates(std::string candidates) = 0;

	};
	class P2pWrapper
	{
	public:
		P2pWrapper();
		~P2pWrapper();
	public:
		static bool InitP2pEnv();
		static void UnInitP2pEnv();

	public:
		bool Start(std::string stun_ip, int stun_port, bool is_controller);
		void Stop();
		
		bool SetRemoteCandidate(std::string candidates);

		int  SendData(const char *buffer, int len);

	private:
		static int  cb_candidate_gather_done(NiceAgent *agent, guint stream_id, gpointer data)
		{
			P2pWrapper *wrapper = (P2pWrapper *)data;
			return wrapper->candidate_gather_done(agent, stream_id);
		}
		static void cb_new_selected_pair(NiceAgent *agent, guint stream_id, guint component_id, gchar *lfoundation, gchar *rfoundation, gpointer data)
		{
			P2pWrapper *wrapper = (P2pWrapper *)data;
			wrapper->new_selected_pair(agent, stream_id, component_id, lfoundation, rfoundation);
		}

		static void cb_component_state_changed(NiceAgent *agent, guint stream_id, guint component_id, guint state, gpointer data)
		{
			P2pWrapper *wrapper = (P2pWrapper *)data;
			wrapper->component_state_changed(agent, stream_id, component_id, state);
		}

		static void cb_nice_recv(NiceAgent *agent, guint stream_id, guint component_id, guint len, gchar *buf, gpointer data)
		{
			P2pWrapper *wrapper = (P2pWrapper *)data;
			wrapper->nice_recv(agent, stream_id, component_id, len, buf);
		}

		static void NetThreadCallback();

	private:
		int  candidate_gather_done(NiceAgent *agent, guint stream_id);
		void new_selected_pair(NiceAgent *agent, guint stream_id, guint component_id, gchar *lfoundation, gchar *rfoundation);
		void component_state_changed(NiceAgent *agent, guint stream_id, guint component_id, guint state);
		void nice_recv(NiceAgent *agent, guint stream_id, guint component_id, guint len, gchar *buf);

	private:
		NiceAgent		*agent_;
		unsigned int	stream_id_;

	public:
		static std::thread  *net_thread_;
		static GMainLoop	*g_main_loop_;
	};
}